#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2018, Olivier Verriest <verri@x25.pm>
# License: https://www.pysnmp.com/pysnmp/license.html
#
import hmac
import sys
from hashlib import sha224, sha256, sha384, sha512

from pyasn1.type import univ
from pysnmp.proto import errind, error
from pysnmp.proto.secmod.rfc3414 import localkey
from pysnmp.proto.secmod.rfc3414.auth import base


# 7.2.4


class HmacSha2(base.AbstractAuthenticationService):
    """The HMAC-SHA-2 authentication protocol.

    https://tools.ietf.org/html/rfc7860#section-7.2.4
    """

    SHA224_SERVICE_ID = (1, 3, 6, 1, 6, 3, 10, 1, 1, 4)  # usmHMAC128SHA224AuthProtocol
    SHA256_SERVICE_ID = (1, 3, 6, 1, 6, 3, 10, 1, 1, 5)  # usmHMAC192SHA256AuthProtocol
    SAH384_SERVICE_ID = (1, 3, 6, 1, 6, 3, 10, 1, 1, 6)  # usmHMAC256SHA384AuthProtocol
    SHA512_SERVICE_ID = (1, 3, 6, 1, 6, 3, 10, 1, 1, 7)  # usmHMAC384SHA512AuthProtocol
    KEY_LENGTH = {
        SHA224_SERVICE_ID: 28,
        SHA256_SERVICE_ID: 32,
        SAH384_SERVICE_ID: 48,
        SHA512_SERVICE_ID: 64,
    }
    DIGEST_LENGTH = {
        SHA224_SERVICE_ID: 16,
        SHA256_SERVICE_ID: 24,
        SAH384_SERVICE_ID: 32,
        SHA512_SERVICE_ID: 48,
    }
    HASH_ALGORITHM = {
        SHA224_SERVICE_ID: sha224,
        SHA256_SERVICE_ID: sha256,
        SAH384_SERVICE_ID: sha384,
        SHA512_SERVICE_ID: sha512,
    }

    IPAD = [0x36] * 64
    OPAD = [0x5C] * 64

    def __init__(self, oid):
        """Create a HMAC-SHA2 authentication service."""
        if oid not in self.HASH_ALGORITHM:
            raise error.ProtocolError(
                f"No SHA-2 authentication algorithm {oid} available"
            )
        self.__hashAlgo = self.HASH_ALGORITHM[oid]
        self.__digestLength = self.DIGEST_LENGTH[oid]
        self.__placeHolder = univ.OctetString((0,) * self.__digestLength).asOctets()

    def hash_passphrase(self, authKey):
        """Hash a passphrase."""
        return localkey.hash_passphrase(authKey, self.__hashAlgo)

    def localize_key(self, authKey, snmpEngineID):
        """Localize a key."""
        return localkey.localize_key(authKey, snmpEngineID, self.__hashAlgo)

    @property
    def digest_length(self):
        """Return digest length."""
        return self.__digestLength

    # 7.3.1
    def authenticate_outgoing_message(self, authKey, wholeMsg):
        """Authenticate outgoing message."""
        # 7.3.1.1
        location = wholeMsg.find(self.__placeHolder)
        if location == -1:
            raise error.ProtocolError("Can't locate digest placeholder")
        wholeHead = wholeMsg[:location]
        wholeTail = wholeMsg[location + self.__digestLength :]

        # 7.3.1.2, 7.3.1.3
        try:
            mac = hmac.new(authKey.asOctets(), wholeMsg, self.__hashAlgo)

        except errind.ErrorIndication:
            raise error.StatusInformation(errorIndication=sys.exc_info()[1])

        # 7.3.1.4
        mac = mac.digest()[: self.__digestLength]

        # 7.3.1.5 & 6
        return wholeHead + mac + wholeTail

    # 7.3.2
    def authenticate_incoming_message(self, authKey, authParameters, wholeMsg):
        """Authenticate incoming message."""
        # 7.3.2.1 & 2
        if len(authParameters) != self.__digestLength:
            raise error.StatusInformation(errorIndication=errind.authenticationError)

        # 7.3.2.3
        location = wholeMsg.find(authParameters.asOctets())
        if location == -1:
            raise error.ProtocolError("Can't locate digest in wholeMsg")
        wholeHead = wholeMsg[:location]
        wholeTail = wholeMsg[location + self.__digestLength :]
        authenticatedWholeMsg = wholeHead + self.__placeHolder + wholeTail

        # 7.3.2.4
        try:
            mac = hmac.new(authKey.asOctets(), authenticatedWholeMsg, self.__hashAlgo)

        except errind.ErrorIndication:
            raise error.StatusInformation(errorIndication=sys.exc_info()[1])

        # 7.3.2.5
        mac = mac.digest()[: self.__digestLength]

        # 7.3.2.6
        if mac != authParameters:
            raise error.StatusInformation(errorIndication=errind.authenticationFailure)

        return authenticatedWholeMsg



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
(MibScalarInstance,) = mibBuilder.import_symbols("SNMPv2-SMI", "MibScalarInstance")

(vacmViewSpinLock,) = mibBuilder.import_symbols(
    "SNMP-VIEW-BASED-ACM-MIB", "vacmViewSpinLock"
)

__vacmViewSpinLock = MibScalarInstance(
    vacmViewSpinLock.name, (0,), vacmViewSpinLock.syntax
)

mibBuilder.export_symbols(
    "__SNMP-VIEW-BASED-ACM-MIB", vacmViewSpinLock=__vacmViewSpinLock
)



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
import asyncio
import socket
from typing import Tuple

from pysnmp.carrier.asyncio.dgram import udp, udp6
from pysnmp.error import PySnmpError
from pysnmp.hlapi.transport import AbstractTransportTarget

__all__ = ["Udp6TransportTarget", "UdpTransportTarget"]


class UdpTransportTarget(AbstractTransportTarget):
    """Represent UDP/IPv4 transport endpoint.

    This object can be used for passing UDP/IPv4 configuration
    information to the
    :py:class:`~pysnmp.hlapi.v1arch.asyncio.AsyncCommandGenerator` and
    :py:class:`~pysnmp.hlapi.v1arch.asyncio.AsyncNotificationOriginator`
    objects scheduled on I/O by :py:class:`~pysnmp.hlapi.SnmpDispatcher`
    class instance.

    See :RFC:`1906#section-3` for more information on the UDP transport mapping.

    Parameters
    ----------
    transportAddr: tuple
        Indicates remote address in Python :py:mod:`socket` module format
        which is a tuple of FQDN, port where FQDN is a string representing
        either hostname or IPv4 address in quad-dotted form, port is an
        integer.
    timeout: :py:class:`int`
        Response timeout in seconds.
    retries: :py:class:`int`
        Maximum number of request retries, 0 retries means just a single
        request.
    tagList: str
        Arbitrary string that contains a list of tag values which are used
        to select target addresses for a particular operation
        (:RFC:`3413#section-4.1.4`).

    Examples
    --------
    >>> from pysnmp.hlapi.v1arch.asyncio import UdpTransportTarget
    >>> await UdpTransportTarget.create(('demo.pysnmp.com', 161))
    UdpTransportTarget(('195.218.195.228', 161), timeout=1, retries=5)
    >>>
    """

    TRANSPORT_DOMAIN: Tuple[int, ...] = udp.DOMAIN_NAME
    PROTO_TRANSPORT = udp.UdpAsyncioTransport

    async def _resolve_address(self, transportAddr: Tuple) -> Tuple[str, int]:
        try:
            loop = asyncio.get_event_loop()
            return (
                await loop.getaddrinfo(
                    transportAddr[0],
                    transportAddr[1],
                    family=socket.AF_INET,
                    type=socket.SOCK_DGRAM,
                    proto=socket.IPPROTO_UDP,
                )
            )[0][4][:2]
        except socket.gaierror as exc:
            raise PySnmpError(
                "Bad IPv4/UDP transport address {}: {}".format(
                    "@".join([str(x) for x in transportAddr]), exc
                )
            )


class Udp6TransportTarget(AbstractTransportTarget):
    """Represent UDP/IPv6 transport endpoint.

    This object can be used for passing UDP/IPv6 configuration
    information to the
    :py:class:`~pysnmp.hlapi.v1arch.asyncio.AsyncCommandGenerator` and
    :py:class:`~pysnmp.hlapi.v1arch.asyncio.AsyncNotificationOriginator`
    objects scheduled on I/O by :py:class:`~pysnmp.hlapi.SnmpDispatcher`
    class instance.

    See :RFC:`1906#section-3`, :RFC:`2851#section-4` for more information
    on the UDP and IPv6 transport mapping.

    Parameters
    ----------
    transportAddr: tuple
        Indicates remote address in Python :py:mod:`socket` module format
        which is a tuple of FQDN, port where FQDN is a string representing
        either hostname or IPv6 address in one of three conventional forms
        (:RFC:`1924#section-3`), port is an integer.
    timeout: int
        Response timeout in seconds.
    retries: int
        Maximum number of request retries, 0 retries means just a single
        request.
    tagList: str
        Arbitrary string that contains a list of tag values which are used
        to select target addresses for a particular operation
        (:RFC:`3413#section-4.1.4`).

    Examples
    --------
    >>> from pysnmp.hlapi.v1arch.asyncio import Udp6TransportTarget
    >>> await Udp6TransportTarget.create(('google.com', 161))
    Udp6TransportTarget(('2a00:1450:4014:80a::100e', 161), timeout=1, retries=5, tagList='')
    >>> await Udp6TransportTarget.create(('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210', 161))
    Udp6TransportTarget(('fedc:ba98:7654:3210:fedc:ba98:7654:3210', 161), timeout=1, retries=5, tagList='')
    >>> await Udp6TransportTarget.create(('1080:0:0:0:8:800:200C:417A', 161))
    Udp6TransportTarget(('1080::8:800:200c:417a', 161), timeout=1, retries=5, tagList='')
    >>> await Udp6TransportTarget.create(('::0', 161))
    Udp6TransportTarget(('::', 161), timeout=1, retries=5, tagList='')
    >>> await Udp6TransportTarget.create(('::', 161))
    Udp6TransportTarget(('::', 161), timeout=1, retries=5, tagList='')
    >>>
    """

    TRANSPORT_DOMAIN: Tuple[int, ...] = udp6.DOMAIN_NAME
    PROTO_TRANSPORT = udp6.Udp6AsyncioTransport

    async def _resolve_address(self, transportAddr: Tuple) -> Tuple[str, int]:
        try:
            loop = asyncio.get_event_loop()
            return (
                await loop.getaddrinfo(
                    transportAddr[0],
                    transportAddr[1],
                    family=socket.AF_INET6,
                    type=socket.SOCK_DGRAM,
                    proto=socket.IPPROTO_UDP,
                )
            )[0][4][:2]
        except socket.gaierror as exc:
            raise PySnmpError(
                "Bad IPv6/UDP transport address {}: {}".format(
                    "@".join([str(x) for x in transportAddr]), exc
                )
            )



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
import warnings


from pysnmp import debug, error
from pysnmp.carrier.asyncio.dgram import udp, udp6
from pysnmp.carrier.base import AbstractTransport
from pysnmp.entity.engine import SnmpEngine
from pysnmp.proto import rfc1902, rfc1905
from pysnmp.proto.secmod.eso.priv import aes192, aes256, des3
from pysnmp.proto.secmod.rfc3414.auth import hmacmd5, hmacsha, noauth
from pysnmp.proto.secmod.rfc3414.priv import des, nopriv
from pysnmp.proto.secmod.rfc3414.service import SnmpUSMSecurityModel
from pysnmp.proto.secmod.rfc3826.priv import aes
from pysnmp.proto.secmod.rfc7860.auth import hmacsha2

# Old to new attribute mapping
deprecated_attributes = {
    "snmpUDPDomain": "SNMP_UDP_DOMAIN",
    "snmpUDP6Domain": "SNMP_UDP6_DOMAIN",
    "snmpLocalDomain": "SNMP_LOCAL_DOMAIN",
    "usmHMACMD5AuthProtocol": "USM_AUTH_HMAC96_MD5",
    "usmHMACSHAAuthProtocol": "USM_AUTH_HMAC96_SHA",
    "usmHMAC128SHA224AuthProtocol": "USM_AUTH_HMAC128_SHA224",
    "usmHMAC192SHA256AuthProtocol": "USM_AUTH_HMAC192_SHA256",
    "usmHMAC256SHA384AuthProtocol": "USM_AUTH_HMAC256_SHA384",
    "usmHMAC384SHA512AuthProtocol": "USM_AUTH_HMAC384_SHA512",
    "usmNoAuthProtocol": "USM_AUTH_NONE",
    "usmDESPrivProtocol": "USM_PRIV_CBC56_DES",
    "usm3DESEDEPrivProtocol": "USM_PRIV_CBC168_3DES",
    "usmAesCfb128Protocol": "USM_PRIV_CFB128_AES",
    "usmAesBlumenthalCfb192Protocol": "USM_PRIV_CFB192_AES_BLUMENTHAL",
    "usmAesBlumenthalCfb256Protocol": "USM_PRIV_CFB256_AES_BLUMENTHAL",
    "usmAesCfb192Protocol": "USM_PRIV_CFB192_AES",
    "usmAesCfb256Protocol": "USM_PRIV_CFB256_AES",
    "usmNoPrivProtocol": "USM_PRIV_NONE",
    "usmKeyTypePassphrase": "USM_KEY_TYPE_PASSPHRASE",
    "usmKeyTypeMaster": "USM_KEY_TYPE_MASTER",
    "usmKeyTypeLocalized": "USM_KEY_TYPE_LOCALIZED",
    "authServices": "AUTH_SERVICES",
    "privServices": "PRIV_SERVICES",
    "addV1System": "add_v1_system",
    "delV1System": "delete_v1_system",
    "addV3User": "add_v3_user",
    "delV3User": "delete_v3_user",
    "addTransport": "add_transport",
    "delTransport": "delete_transport",
    "addTargetParams": "add_target_parameters",
    "delTargetParams": "delete_target_parameters",
    "addTargetAddr": "add_target_address",
    "delTargetAddr": "delete_target_address",
    "addVacmUser": "add_vacm_user",
    "delVacmUser": "delete_vacm_user",
}


def __getattr__(attr: str):
    if new_attr := deprecated_attributes.get(attr):
        warnings.warn(
            f"{attr} is deprecated. Please use {new_attr} instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return globals()[new_attr]
    raise AttributeError(f"module '{__name__}' has no attribute '{attr}'")


# A shortcut to popular constants

# Transports
SNMP_UDP_DOMAIN = udp.SNMP_UDP_DOMAIN
SNMP_UDP6_DOMAIN = udp6.SNMP_UDP6_DOMAIN

# Auth protocol
USM_AUTH_HMAC96_MD5 = hmacmd5.HmacMd5.SERVICE_ID
USM_AUTH_HMAC96_SHA = hmacsha.HmacSha.SERVICE_ID
USM_AUTH_HMAC128_SHA224 = hmacsha2.HmacSha2.SHA224_SERVICE_ID
USM_AUTH_HMAC192_SHA256 = hmacsha2.HmacSha2.SHA256_SERVICE_ID
USM_AUTH_HMAC256_SHA384 = hmacsha2.HmacSha2.SAH384_SERVICE_ID
USM_AUTH_HMAC384_SHA512 = hmacsha2.HmacSha2.SHA512_SERVICE_ID

USM_AUTH_NONE = noauth.NoAuth.SERVICE_ID
"""No authentication service"""

# Privacy protocol
USM_PRIV_CBC56_DES = des.Des.SERVICE_ID
USM_PRIV_CBC168_3DES = des3.Des3.SERVICE_ID
USM_PRIV_CFB128_AES = aes.Aes.SERVICE_ID
USM_PRIV_CFB192_AES_BLUMENTHAL = (
    aes192.AesBlumenthal192.SERVICE_ID
)  # semi-standard but not widely used
USM_PRIV_CFB256_AES_BLUMENTHAL = (
    aes256.AesBlumenthal256.SERVICE_ID
)  # semi-standard but not widely used
USM_PRIV_CFB192_AES = aes192.Aes192.SERVICE_ID  # non-standard but used by many vendors
USM_PRIV_CFB256_AES = aes256.Aes256.SERVICE_ID  # non-standard but used by many vendors
USM_PRIV_NONE = nopriv.NoPriv.SERVICE_ID

# USM key types (PYSNMP-USM-MIB::pysnmpUsmKeyType)
USM_KEY_TYPE_PASSPHRASE = 0
USM_KEY_TYPE_MASTER = 1
USM_KEY_TYPE_LOCALIZED = 2

# Auth services
AUTH_SERVICES = SnmpUSMSecurityModel.AUTH_SERVICES

# Privacy services
PRIV_SERVICES = SnmpUSMSecurityModel.PRIV_SERVICES


def __cook_v1_system_info(snmpEngine: SnmpEngine, communityIndex):
    mibBuilder = snmpEngine.get_mib_builder()

    (snmpEngineID,) = mibBuilder.import_symbols("__SNMP-FRAMEWORK-MIB", "snmpEngineID")  # type: ignore
    (snmpCommunityEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-COMMUNITY-MIB", "snmpCommunityEntry"
    )
    tblIdx = snmpCommunityEntry.getInstIdFromIndices(communityIndex)
    return snmpCommunityEntry, tblIdx, snmpEngineID


def add_v1_system(
    snmpEngine: SnmpEngine,
    communityIndex: str,
    communityName: str,
    contextEngineId=None,
    contextName=None,
    transportTag=None,
    securityName=None,
):
    """Register SNMPv1 and v2c community."""
    (snmpCommunityEntry, tblIdx, snmpEngineID) = __cook_v1_system_info(
        snmpEngine, communityIndex
    )

    if contextEngineId is None:
        contextEngineId = snmpEngineID.syntax
    else:
        contextEngineId = snmpEngineID.syntax.clone(contextEngineId)

    if contextName is None:
        contextName = b""

    securityName = securityName is not None and securityName or communityIndex

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpCommunityEntry.name + (8,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpCommunityEntry.name + (1,) + tblIdx, communityIndex),
        (snmpCommunityEntry.name + (2,) + tblIdx, communityName),
        (
            snmpCommunityEntry.name + (3,) + tblIdx,
            securityName is not None and securityName or communityIndex,
        ),
        (snmpCommunityEntry.name + (4,) + tblIdx, contextEngineId),
        (snmpCommunityEntry.name + (5,) + tblIdx, contextName),
        (snmpCommunityEntry.name + (6,) + tblIdx, transportTag),
        (snmpCommunityEntry.name + (7,) + tblIdx, "nonVolatile"),
        (snmpCommunityEntry.name + (8,) + tblIdx, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )

    debug.logger & debug.FLAG_SM and debug.logger(
        "addV1System: added new table entry "
        'communityIndex "%s" communityName "%s" securityName "%s" '
        'contextEngineId "%s" contextName "%s" transportTag '
        '"%s"'
        % (
            communityIndex,
            communityName,
            securityName,
            contextEngineId,
            contextName,
            transportTag,
        )
    )


def delete_v1_system(snmpEngine: SnmpEngine, communityIndex):
    """Unregister SNMPv1 and v2c community."""
    (snmpCommunityEntry, tblIdx, snmpEngineID) = __cook_v1_system_info(
        snmpEngine, communityIndex
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpCommunityEntry.name + (8,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )

    debug.logger & debug.FLAG_SM and debug.logger(
        "delV1System: deleted table entry by communityIndex " '"%s"' % (communityIndex,)
    )


def __cook_v3_user_info(snmpEngine: SnmpEngine, securityName, securityEngineId):
    mibBuilder = snmpEngine.get_mib_builder()

    (snmpEngineID,) = mibBuilder.import_symbols("__SNMP-FRAMEWORK-MIB", "snmpEngineID")  # type: ignore

    if securityEngineId is None:
        securityEngineId = snmpEngineID.syntax
    else:
        securityEngineId = snmpEngineID.syntax.clone(securityEngineId)

    (usmUserEntry,) = mibBuilder.import_symbols("SNMP-USER-BASED-SM-MIB", "usmUserEntry")  # type: ignore
    tblIdx1 = usmUserEntry.getInstIdFromIndices(securityEngineId, securityName)

    (pysnmpUsmSecretEntry,) = mibBuilder.import_symbols(  # type: ignore
        "PYSNMP-USM-MIB", "pysnmpUsmSecretEntry"
    )
    tblIdx2 = pysnmpUsmSecretEntry.getInstIdFromIndices(securityName)

    return securityEngineId, usmUserEntry, tblIdx1, pysnmpUsmSecretEntry, tblIdx2


def add_v3_user(
    snmpEngine: SnmpEngine,
    userName,
    authProtocol=USM_AUTH_NONE,
    authKey=None,
    privProtocol=USM_PRIV_NONE,
    privKey=None,
    securityEngineId=None,
    securityName=None,
    authKeyType=USM_KEY_TYPE_PASSPHRASE,
    privKeyType=USM_KEY_TYPE_PASSPHRASE,
):
    """Register SNMPv3 user."""
    mibBuilder = snmpEngine.get_mib_builder()

    if securityName is None:
        securityName = userName

    (
        securityEngineId,
        usmUserEntry,
        tblIdx1,
        pysnmpUsmSecretEntry,
        tblIdx2,
    ) = __cook_v3_user_info(snmpEngine, securityName, securityEngineId)

    # Load augmenting table before creating new row in base one
    (pysnmpUsmKeyEntry,) = mibBuilder.import_symbols(  # type: ignore
        "PYSNMP-USM-MIB", "pysnmpUsmKeyEntry"
    )

    # Load clone-from (may not be needed)
    (zeroDotZero,) = mibBuilder.import_symbols("SNMPv2-SMI", "zeroDotZero")  # type: ignore

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (usmUserEntry.name + (13,) + tblIdx1, "destroy"), **dict(snmpEngine=snmpEngine)
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (usmUserEntry.name + (2,) + tblIdx1, userName),
        (usmUserEntry.name + (3,) + tblIdx1, securityName),
        (usmUserEntry.name + (4,) + tblIdx1, zeroDotZero.name),
        (usmUserEntry.name + (5,) + tblIdx1, authProtocol),
        (usmUserEntry.name + (8,) + tblIdx1, privProtocol),
        (usmUserEntry.name + (13,) + tblIdx1, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )

    if authProtocol not in AUTH_SERVICES:
        raise error.PySnmpError(f"Unknown auth protocol {authProtocol}")

    if privProtocol not in PRIV_SERVICES:
        raise error.PySnmpError(f"Unknown privacy protocol {privProtocol}")

    (pysnmpUsmKeyType,) = mibBuilder.import_symbols(  # type: ignore
        "__PYSNMP-USM-MIB", "pysnmpUsmKeyType"
    )

    authKeyType = pysnmpUsmKeyType.syntax.clone(authKeyType)

    # Localize authentication key unless given

    authKey = authKey and rfc1902.OctetString(authKey)

    masterAuthKey = localAuthKey = authKey

    if authKeyType < USM_KEY_TYPE_MASTER:  # pass phrase is given
        masterAuthKey = AUTH_SERVICES[authProtocol].hash_passphrase(authKey or b"")

    if authKeyType < USM_KEY_TYPE_LOCALIZED:  # pass phrase or master key is given
        localAuthKey = AUTH_SERVICES[authProtocol].localize_key(
            masterAuthKey, securityEngineId
        )

    # Localize privacy key unless given

    privKeyType = pysnmpUsmKeyType.syntax.clone(privKeyType)

    privKey = privKey and rfc1902.OctetString(privKey)

    masterPrivKey = localPrivKey = privKey

    if privKeyType < USM_KEY_TYPE_MASTER:  # pass phrase is given
        masterPrivKey = PRIV_SERVICES[privProtocol].hash_passphrase(
            authProtocol, privKey or b""
        )

    if privKeyType < USM_KEY_TYPE_LOCALIZED:  # pass phrase or master key is given
        localPrivKey = PRIV_SERVICES[privProtocol].localize_key(
            authProtocol, masterPrivKey, securityEngineId
        )

    # Commit only the keys we have

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (pysnmpUsmKeyEntry.name + (1,) + tblIdx1, localAuthKey),
        (pysnmpUsmKeyEntry.name + (2,) + tblIdx1, localPrivKey),
        **dict(snmpEngine=snmpEngine),
    )

    if authKeyType < USM_KEY_TYPE_LOCALIZED:
        snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
            (pysnmpUsmKeyEntry.name + (3,) + tblIdx1, masterAuthKey),
            **dict(snmpEngine=snmpEngine),
        )

    if privKeyType < USM_KEY_TYPE_LOCALIZED:
        snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
            (pysnmpUsmKeyEntry.name + (4,) + tblIdx1, masterPrivKey),
            **dict(snmpEngine=snmpEngine),
        )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (pysnmpUsmSecretEntry.name + (4,) + tblIdx2, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )

    # Commit plain-text pass-phrases if we have them

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (pysnmpUsmSecretEntry.name + (4,) + tblIdx2, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )

    if authKeyType < USM_KEY_TYPE_MASTER:
        snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
            (pysnmpUsmSecretEntry.name + (1,) + tblIdx2, userName),
            (pysnmpUsmSecretEntry.name + (2,) + tblIdx2, authKey),
            **dict(snmpEngine=snmpEngine),
        )

    if privKeyType < USM_KEY_TYPE_MASTER:
        snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
            (pysnmpUsmSecretEntry.name + (1,) + tblIdx2, userName),
            (pysnmpUsmSecretEntry.name + (3,) + tblIdx2, privKey),
            **dict(snmpEngine=snmpEngine),
        )

    debug.logger & debug.FLAG_SM and debug.logger(
        "addV3User: added new table entries "
        'userName "%s" securityName "%s" authProtocol %s '
        'privProtocol %s localAuthKey "%s" localPrivKey "%s" '
        'masterAuthKey "%s" masterPrivKey "%s" authKey "%s" '
        'privKey "%s" by index securityName "%s" securityEngineId '
        '"%s"'
        % (
            userName,
            securityName,
            authProtocol,
            privProtocol,
            localAuthKey and localAuthKey.prettyPrint(),
            localPrivKey and localPrivKey.prettyPrint(),
            masterAuthKey and masterAuthKey.prettyPrint(),
            masterPrivKey and masterPrivKey.prettyPrint(),
            authKey and authKey.prettyPrint(),
            privKey and privKey.prettyPrint(),
            securityName,
            securityEngineId.prettyPrint(),
        )
    )


def delete_v3_user(
    snmpEngine: SnmpEngine,
    userName,
    securityEngineId=None,
):
    """Unregister SNMPv3 user."""
    (
        securityEngineId,
        usmUserEntry,
        tblIdx1,
        pysnmpUsmSecretEntry,
        tblIdx2,
    ) = __cook_v3_user_info(snmpEngine, userName, securityEngineId)

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (usmUserEntry.name + (13,) + tblIdx1, "destroy"), **dict(snmpEngine=snmpEngine)
    )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (pysnmpUsmSecretEntry.name + (4,) + tblIdx2, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )

    debug.logger & debug.FLAG_SM and debug.logger(
        "delV3User: deleted table entries by index "
        'userName "%s" securityEngineId '
        '"%s"' % (userName, securityEngineId.prettyPrint())
    )

    # Drop all derived rows
    varBinds = initialVarBinds = (
        (usmUserEntry.name + (1,), None),  # usmUserEngineID
        (usmUserEntry.name + (2,), None),  # usmUserName
        (usmUserEntry.name + (4,), None),  # usmUserCloneFrom
    )

    while varBinds:
        varBinds = (
            snmpEngine.message_dispatcher.mib_instrum_controller.read_next_variables(
                *varBinds, **dict(snmpEngine=snmpEngine)
            )
        )
        if varBinds[0][1].isSameTypeWith(rfc1905.endOfMibView):
            break
        if varBinds[0][0][: len(initialVarBinds[0][0])] != initialVarBinds[0][0]:
            break
        elif varBinds[2][1] == tblIdx1:  # cloned from this entry
            delete_v3_user(snmpEngine, varBinds[1][1], varBinds[0][1])
            varBinds = initialVarBinds


def __cook_target_parameters_info(snmpEngine: SnmpEngine, name):
    mibBuilder = snmpEngine.get_mib_builder()

    (snmpTargetParamsEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-TARGET-MIB", "snmpTargetParamsEntry"
    )
    tblIdx = snmpTargetParamsEntry.getInstIdFromIndices(name)
    return snmpTargetParamsEntry, tblIdx


# mpModel: 0 == SNMPv1, 1 == SNMPv2c, 3 == SNMPv3
def add_target_parameters(
    snmpEngine: SnmpEngine, name, securityName, securityLevel, mpModel=3
):
    """Register target parameters."""
    if mpModel == 0:
        securityModel = 1
    elif mpModel in (1, 2):
        securityModel = 2
    elif mpModel == 3:
        securityModel = 3
    else:
        raise error.PySnmpError("Unknown MP model %s" % mpModel)

    snmpTargetParamsEntry, tblIdx = __cook_target_parameters_info(snmpEngine, name)

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpTargetParamsEntry.name + (7,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpTargetParamsEntry.name + (1,) + tblIdx, name),
        (snmpTargetParamsEntry.name + (2,) + tblIdx, mpModel),
        (snmpTargetParamsEntry.name + (3,) + tblIdx, securityModel),
        (snmpTargetParamsEntry.name + (4,) + tblIdx, securityName),
        (snmpTargetParamsEntry.name + (5,) + tblIdx, securityLevel),
        (snmpTargetParamsEntry.name + (7,) + tblIdx, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )


def delete_target_parameters(snmpEngine: SnmpEngine, name: str):
    """Delete target parameters."""
    snmpTargetParamsEntry, tblIdx = __cook_target_parameters_info(snmpEngine, name)
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpTargetParamsEntry.name + (7,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )


def __cook_target_address_info(snmpEngine: SnmpEngine, addrName: str):
    mibBuilder = snmpEngine.get_mib_builder()

    (snmpTargetAddrEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-TARGET-MIB", "snmpTargetAddrEntry"
    )
    (snmpSourceAddrEntry,) = mibBuilder.import_symbols(  # type: ignore
        "PYSNMP-SOURCE-MIB", "snmpSourceAddrEntry"
    )
    tblIdx = snmpTargetAddrEntry.getInstIdFromIndices(addrName)
    return snmpTargetAddrEntry, snmpSourceAddrEntry, tblIdx


def add_target_address(
    snmpEngine: SnmpEngine,
    addrName: str,
    transportDomain: "tuple[int, ...]",
    transportAddress: "tuple[str, int]",
    params: str,
    timeout: "float | None" = None,
    retryCount: "int | None" = None,
    tagList=b"",
    sourceAddress=None,
):
    """Register target address."""
    mibBuilder = snmpEngine.get_mib_builder()

    (snmpTargetAddrEntry, snmpSourceAddrEntry, tblIdx) = __cook_target_address_info(
        snmpEngine, addrName
    )

    if transportDomain[: len(SNMP_UDP_DOMAIN)] == SNMP_UDP_DOMAIN:
        (SnmpUDPAddress,) = mibBuilder.import_symbols("SNMPv2-TM", "SnmpUDPAddress")  # type: ignore
        transportAddress = SnmpUDPAddress(transportAddress)
        if sourceAddress is None:
            sourceAddress = ("0.0.0.0", 0)
        sourceAddress = SnmpUDPAddress(sourceAddress)
    elif transportDomain[: len(SNMP_UDP6_DOMAIN)] == SNMP_UDP6_DOMAIN:
        (TransportAddressIPv6,) = mibBuilder.import_symbols(  # type: ignore
            "TRANSPORT-ADDRESS-MIB", "TransportAddressIPv6"
        )
        transportAddress = TransportAddressIPv6(transportAddress)
        if sourceAddress is None:
            sourceAddress = ("::", 0)
        sourceAddress = TransportAddressIPv6(sourceAddress)

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpTargetAddrEntry.name + (9,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpTargetAddrEntry.name + (1,) + tblIdx, addrName),
        (snmpTargetAddrEntry.name + (2,) + tblIdx, transportDomain),
        (snmpTargetAddrEntry.name + (3,) + tblIdx, transportAddress),
        (snmpTargetAddrEntry.name + (4,) + tblIdx, timeout),
        (snmpTargetAddrEntry.name + (5,) + tblIdx, retryCount),
        (snmpTargetAddrEntry.name + (6,) + tblIdx, tagList),
        (snmpTargetAddrEntry.name + (7,) + tblIdx, params),
        (snmpSourceAddrEntry.name + (1,) + tblIdx, sourceAddress),
        (snmpTargetAddrEntry.name + (9,) + tblIdx, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )


def delete_target_address(snmpEngine: SnmpEngine, addrName: str):
    """Delete target address."""
    (snmpTargetAddrEntry, snmpSourceAddrEntry, tblIdx) = __cook_target_address_info(
        snmpEngine, addrName
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpTargetAddrEntry.name + (9,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )


def add_transport(
    snmpEngine: SnmpEngine,
    transportDomain: "tuple[int, ...]",
    transport: AbstractTransport,
):
    """Register transport with dispatcher."""
    if snmpEngine.transport_dispatcher:
        if not transport.is_compatible_with_dispatcher(snmpEngine.transport_dispatcher):
            raise error.PySnmpError(
                f"Transport {transport!r} is not compatible with dispatcher {snmpEngine.transport_dispatcher!r}"
            )
    else:
        protoTransportDispatcher = transport.PROTO_TRANSPORT_DISPATCHER
        if protoTransportDispatcher is not None:
            snmpEngine.register_transport_dispatcher(protoTransportDispatcher())
            # here we note that we have created transportDispatcher automatically
            snmpEngine.set_user_context(automaticTransportDispatcher=0)

    if snmpEngine.transport_dispatcher:
        snmpEngine.transport_dispatcher.register_transport(transportDomain, transport)
        automaticTransportDispatcher = snmpEngine.get_user_context(
            "automaticTransportDispatcher"
        )
        if automaticTransportDispatcher is not None:
            snmpEngine.set_user_context(
                automaticTransportDispatcher=automaticTransportDispatcher + 1
            )


def get_transport(snmpEngine: SnmpEngine, transportDomain: "tuple[int, ...]"):
    """Return transport from dispatcher."""
    if not snmpEngine.transport_dispatcher:
        return
    try:
        return snmpEngine.transport_dispatcher.get_transport(transportDomain)
    except error.PySnmpError:
        return


def delete_transport(snmpEngine: SnmpEngine, transportDomain: "tuple[int, ...]"):
    """Remove transport from dispatcher."""
    if not snmpEngine.transport_dispatcher:
        return
    transport = get_transport(snmpEngine, transportDomain)
    snmpEngine.transport_dispatcher.unregister_transport(transportDomain)
    # automatically shutdown automatically created transportDispatcher
    automaticTransportDispatcher = snmpEngine.get_user_context(
        "automaticTransportDispatcher"
    )
    if automaticTransportDispatcher is not None:
        automaticTransportDispatcher -= 1
        snmpEngine.set_user_context(
            automaticTransportDispatcher=automaticTransportDispatcher
        )
        if not automaticTransportDispatcher:
            snmpEngine.close_dispatcher()
            snmpEngine.delete_user_context(automaticTransportDispatcher)
    return transport


addSocketTransport = add_transport  # noqa: N816
delSocketTransport = delete_transport  # noqa: N816


# VACM shortcuts


def __cook_vacm_context_info(snmpEngine: SnmpEngine, contextName):
    mibBuilder = snmpEngine.get_mib_builder()
    (vacmContextEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-VIEW-BASED-ACM-MIB", "vacmContextEntry"
    )
    tblIdx = vacmContextEntry.getInstIdFromIndices(contextName)
    return vacmContextEntry, tblIdx


def add_context(snmpEngine: SnmpEngine, contextName):
    """Setup VACM context."""
    vacmContextEntry, tblIdx = __cook_vacm_context_info(snmpEngine, contextName)

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmContextEntry.name + (2,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmContextEntry.name + (1,) + tblIdx, contextName),
        (vacmContextEntry.name + (2,) + tblIdx, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )


def delete_context(snmpEngine: SnmpEngine, contextName):
    """Delete VACM context."""
    vacmContextEntry, tblIdx = __cook_vacm_context_info(snmpEngine, contextName)

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmContextEntry.name + (2,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )


def __cook_vacm_group_info(snmpEngine: SnmpEngine, securityModel, securityName):
    mibBuilder = snmpEngine.get_mib_builder()

    (vacmSecurityToGroupEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-VIEW-BASED-ACM-MIB", "vacmSecurityToGroupEntry"
    )
    tblIdx = vacmSecurityToGroupEntry.getInstIdFromIndices(securityModel, securityName)
    return vacmSecurityToGroupEntry, tblIdx


def add_vacm_group(snmpEngine: SnmpEngine, groupName, securityModel, securityName):
    """Setup VACM group."""
    (vacmSecurityToGroupEntry, tblIdx) = __cook_vacm_group_info(
        snmpEngine, securityModel, securityName
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmSecurityToGroupEntry.name + (5,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmSecurityToGroupEntry.name + (1,) + tblIdx, securityModel),
        (vacmSecurityToGroupEntry.name + (2,) + tblIdx, securityName),
        (vacmSecurityToGroupEntry.name + (3,) + tblIdx, groupName),
        (vacmSecurityToGroupEntry.name + (5,) + tblIdx, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )


def delete_vacm_group(snmpEngine: SnmpEngine, securityModel, securityName):
    """Delete VACM group."""
    vacmSecurityToGroupEntry, tblIdx = __cook_vacm_group_info(
        snmpEngine, securityModel, securityName
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmSecurityToGroupEntry.name + (5,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )


def __cook_vacm_access_info(
    snmpEngine: SnmpEngine, groupName, contextName, securityModel, securityLevel
):
    mibBuilder = snmpEngine.get_mib_builder()

    (vacmAccessEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-VIEW-BASED-ACM-MIB", "vacmAccessEntry"
    )
    tblIdx = vacmAccessEntry.getInstIdFromIndices(
        groupName, contextName, securityModel, securityLevel
    )
    return vacmAccessEntry, tblIdx


def add_vacm_access(
    snmpEngine: SnmpEngine,
    groupName,
    contextPrefix,
    securityModel,
    securityLevel,
    contextMatch,
    readView,
    writeView,
    notifyView,
):
    """Setup VACM access."""
    vacmAccessEntry, tblIdx = __cook_vacm_access_info(
        snmpEngine, groupName, contextPrefix, securityModel, securityLevel
    )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmAccessEntry.name + (9,) + tblIdx, "destroy"), **dict(snmpEngine=snmpEngine)
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmAccessEntry.name + (1,) + tblIdx, contextPrefix),
        (vacmAccessEntry.name + (2,) + tblIdx, securityModel),
        (vacmAccessEntry.name + (3,) + tblIdx, securityLevel),
        (vacmAccessEntry.name + (4,) + tblIdx, contextMatch),
        (vacmAccessEntry.name + (5,) + tblIdx, readView),
        (vacmAccessEntry.name + (6,) + tblIdx, writeView),
        (vacmAccessEntry.name + (7,) + tblIdx, notifyView),
        (vacmAccessEntry.name + (9,) + tblIdx, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )


def delete_vacm_access(
    snmpEngine: SnmpEngine, groupName, contextPrefix, securityModel, securityLevel
):
    """Delete VACM access."""
    vacmAccessEntry, tblIdx = __cook_vacm_access_info(
        snmpEngine, groupName, contextPrefix, securityModel, securityLevel
    )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmAccessEntry.name + (9,) + tblIdx, "destroy"), **dict(snmpEngine=snmpEngine)
    )


def __cook_vacm_view_info(snmpEngine: SnmpEngine, viewName, subTree):
    mibBuilder = snmpEngine.get_mib_builder()

    (vacmViewTreeFamilyEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-VIEW-BASED-ACM-MIB", "vacmViewTreeFamilyEntry"
    )
    tblIdx = vacmViewTreeFamilyEntry.getInstIdFromIndices(viewName, subTree)
    return vacmViewTreeFamilyEntry, tblIdx


def add_vacm_view(snmpEngine: SnmpEngine, viewName, viewType, subTree, subTreeMask):
    """Setup VACM view."""
    vacmViewTreeFamilyEntry, tblIdx = __cook_vacm_view_info(
        snmpEngine, viewName, subTree
    )

    # Allow bitmask specification in form of an OID
    if rfc1902.OctetString(".").asOctets() in rfc1902.OctetString(subTreeMask):
        subTreeMask = rfc1902.ObjectIdentifier(subTreeMask)

    if isinstance(subTreeMask, rfc1902.ObjectIdentifier):
        subTreeMask = tuple(subTreeMask)
        if len(subTreeMask) < len(subTree):
            subTreeMask += (1,) * (len(subTree) - len(subTreeMask))

        subTreeMask = rfc1902.OctetString.fromBinaryString(
            "".join(str(x) for x in subTreeMask)
        )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmViewTreeFamilyEntry.name + (6,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmViewTreeFamilyEntry.name + (1,) + tblIdx, viewName),
        (vacmViewTreeFamilyEntry.name + (2,) + tblIdx, subTree),
        (vacmViewTreeFamilyEntry.name + (3,) + tblIdx, subTreeMask),
        (vacmViewTreeFamilyEntry.name + (4,) + tblIdx, viewType),
        (vacmViewTreeFamilyEntry.name + (6,) + tblIdx, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )


def delete_vacm_view(snmpEngine: SnmpEngine, viewName, subTree):
    """Delete VACM view."""
    vacmViewTreeFamilyEntry, tblIdx = __cook_vacm_view_info(
        snmpEngine, viewName, subTree
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (vacmViewTreeFamilyEntry.name + (6,) + tblIdx, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )


# VACM simplicity wrappers


def __cook_vacm_user_info(
    snmpEngine: SnmpEngine, securityModel, securityName, securityLevel
):
    mibBuilder = snmpEngine.get_mib_builder()

    groupName = "v-%s-%d" % (hash(securityName), securityModel)
    (SnmpSecurityLevel,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-FRAMEWORK-MIB", "SnmpSecurityLevel"
    )
    securityLevel = SnmpSecurityLevel(securityLevel)
    return (groupName, securityLevel, "r" + groupName, "w" + groupName, "n" + groupName)


def add_vacm_user(
    snmpEngine: SnmpEngine,
    securityModel: int,
    securityName: str,
    securityLevel: str,
    readSubTree=(),
    writeSubTree=(),
    notifySubTree=(),
    contextName=b"",
):
    """Setup VACM user."""
    (groupName, securityLevel, readView, writeView, notifyView) = __cook_vacm_user_info(
        snmpEngine, securityModel, securityName, securityLevel
    )
    add_context(snmpEngine, contextName)
    add_vacm_group(snmpEngine, groupName, securityModel, securityName)
    add_vacm_access(
        snmpEngine,
        groupName,
        contextName,
        securityModel,
        securityLevel,
        "exact",
        readView,
        writeView,
        notifyView,
    )
    if readSubTree:
        add_vacm_view(snmpEngine, readView, "included", readSubTree, b"")
    if writeSubTree:
        add_vacm_view(snmpEngine, writeView, "included", writeSubTree, b"")
    if notifySubTree:
        add_vacm_view(snmpEngine, notifyView, "included", notifySubTree, b"")


def delete_vacm_user(
    snmpEngine: SnmpEngine,
    securityModel,
    securityName,
    securityLevel,
    readSubTree=(),
    writeSubTree=(),
    notifySubTree=(),
    contextName=b"",
):
    """Delete VACM user."""
    (groupName, securityLevel, readView, writeView, notifyView) = __cook_vacm_user_info(
        snmpEngine, securityModel, securityName, securityLevel
    )
    delete_context(snmpEngine, contextName)
    delete_vacm_group(snmpEngine, securityModel, securityName)
    delete_vacm_access(snmpEngine, groupName, contextName, securityModel, securityLevel)
    if readSubTree:
        delete_vacm_view(snmpEngine, readView, readSubTree)
    if writeSubTree:
        delete_vacm_view(snmpEngine, writeView, writeSubTree)
    if notifySubTree:
        delete_vacm_view(snmpEngine, notifyView, notifySubTree)


# Notification target setup


def __cook_notification_target_info(
    snmpEngine: SnmpEngine, notificationName, paramsName, filterSubtree=None
):
    mibBuilder = snmpEngine.get_mib_builder()

    (snmpNotifyEntry,) = mibBuilder.import_symbols(  # type: ignore
        "SNMP-NOTIFICATION-MIB", "snmpNotifyEntry"
    )
    tblIdx1 = snmpNotifyEntry.getInstIdFromIndices(notificationName)

    (snmpNotifyFilterProfileEntry,) = mibBuilder.import_symbols(
        "SNMP-NOTIFICATION-MIB", "snmpNotifyFilterProfileEntry"
    )
    tblIdx2 = snmpNotifyFilterProfileEntry.getInstIdFromIndices(paramsName)

    profileName = "%s-filter" % hash(notificationName)

    if filterSubtree:
        (snmpNotifyFilterEntry,) = mibBuilder.import_symbols(
            "SNMP-NOTIFICATION-MIB", "snmpNotifyFilterEntry"
        )
        tblIdx3 = snmpNotifyFilterEntry.getInstIdFromIndices(profileName, filterSubtree)
    else:
        snmpNotifyFilterEntry = tblIdx3 = None

    return (
        snmpNotifyEntry,
        tblIdx1,
        snmpNotifyFilterProfileEntry,
        tblIdx2,
        profileName,
        snmpNotifyFilterEntry,
        tblIdx3,
    )


def add_notification_target(
    snmpEngine: SnmpEngine,
    notificationName,
    paramsName,
    transportTag,
    notifyType=None,
    filterSubtree=None,
    filterMask=None,
    filterType=None,
):
    """Setup notification target."""
    (
        snmpNotifyEntry,
        tblIdx1,
        snmpNotifyFilterProfileEntry,
        tblIdx2,
        profileName,
        snmpNotifyFilterEntry,
        tblIdx3,
    ) = __cook_notification_target_info(
        snmpEngine, notificationName, paramsName, filterSubtree
    )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyEntry.name + (5,) + tblIdx1, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyEntry.name + (2,) + tblIdx1, transportTag),
        (snmpNotifyEntry.name + (3,) + tblIdx1, notifyType),
        (snmpNotifyEntry.name + (5,) + tblIdx1, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyFilterProfileEntry.name + (1,) + tblIdx2, profileName),
        (snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )

    if not snmpNotifyFilterEntry:
        return

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyFilterEntry.name + (5,) + tblIdx3, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )
    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyFilterEntry.name + (1,) + tblIdx3, filterSubtree),
        (snmpNotifyFilterEntry.name + (2,) + tblIdx3, filterMask),
        (snmpNotifyFilterEntry.name + (3,) + tblIdx3, filterType),
        (snmpNotifyFilterEntry.name + (5,) + tblIdx3, "createAndGo"),
        **dict(snmpEngine=snmpEngine),
    )


def delete_notification_target(
    snmpEngine: SnmpEngine, notificationName, paramsName, filterSubtree=None
):
    """Remove notification target."""
    (
        snmpNotifyEntry,
        tblIdx1,
        snmpNotifyFilterProfileEntry,
        tblIdx2,
        profileName,
        snmpNotifyFilterEntry,
        tblIdx3,
    ) = __cook_notification_target_info(
        snmpEngine, notificationName, paramsName, filterSubtree
    )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyEntry.name + (5,) + tblIdx1, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )

    if not snmpNotifyFilterEntry:
        return

    snmpEngine.message_dispatcher.mib_instrum_controller.write_variables(
        (snmpNotifyFilterEntry.name + (5,) + tblIdx3, "destroy"),
        **dict(snmpEngine=snmpEngine),
    )


# rfc3415: A.1
def set_initial_vacm_parameters(snmpEngine: SnmpEngine):
    """Set initial VACM parameters as per SNMP-FRAMEWORK-MIB."""
    # rfc3415: A.1.1 --> initial-semi-security-configuration

    # rfc3415: A.1.2
    add_context(snmpEngine, "")

    # rfc3415: A.1.3
    add_vacm_group(snmpEngine, "initial", 3, "initial")

    # rfc3415: A.1.4
    add_vacm_access(
        snmpEngine,
        "initial",
        "",
        3,
        "noAuthNoPriv",
        "exact",
        "restricted",
        None,
        "restricted",
    )
    add_vacm_access(
        snmpEngine,
        "initial",
        "",
        3,
        "authNoPriv",
        "exact",
        "internet",
        "internet",
        "internet",
    )
    add_vacm_access(
        snmpEngine,
        "initial",
        "",
        3,
        "authPriv",
        "exact",
        "internet",
        "internet",
        "internet",
    )

    # rfc3415: A.1.5 (semi-secure)
    add_vacm_view(snmpEngine, "internet", "included", (1, 3, 6, 1), "")
    add_vacm_view(snmpEngine, "restricted", "included", (1, 3, 6, 1, 2, 1, 1), "")
    add_vacm_view(snmpEngine, "restricted", "included", (1, 3, 6, 1, 2, 1, 11), "")
    add_vacm_view(
        snmpEngine, "restricted", "included", (1, 3, 6, 1, 6, 3, 10, 2, 1), ""
    )
    add_vacm_view(
        snmpEngine, "restricted", "included", (1, 3, 6, 1, 6, 3, 11, 2, 1), ""
    )
    add_vacm_view(
        snmpEngine, "restricted", "included", (1, 3, 6, 1, 6, 3, 15, 1, 1), ""
    )



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
(MibScalarInstance,) = mibBuilder.import_symbols("SNMPv2-SMI", "MibScalarInstance")

(
    pysnmpUsmDiscoverable,
    pysnmpUsmDiscovery,
    pysnmpUsmKeyType,
) = mibBuilder.import_symbols(
    "PYSNMP-USM-MIB", "pysnmpUsmDiscoverable", "pysnmpUsmDiscovery", "pysnmpUsmKeyType"
)

__pysnmpUsmDiscoverable = MibScalarInstance(
    pysnmpUsmDiscoverable.name, (0,), pysnmpUsmDiscoverable.syntax
)
__pysnmpUsmDiscovery = MibScalarInstance(
    pysnmpUsmDiscovery.name, (0,), pysnmpUsmDiscovery.syntax
)
__pysnmpUsmKeyType = MibScalarInstance(
    pysnmpUsmKeyType.name, (0,), pysnmpUsmKeyType.syntax
)

mibBuilder.export_symbols(
    "__PYSNMP-USM-MIB",
    pysnmpUsmDiscoverable=__pysnmpUsmDiscoverable,
    pysnmpUsmDiscovery=__pysnmpUsmDiscovery,
    pysnmpUsmKeyType=__pysnmpUsmKeyType,
)



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
__all__ = ["ContextData"]


class ContextData:
    """Creates UDP/IPv6 configuration entry and initialize socket API if needed.

    This object can be used by
    :py:class:`~pysnmp.hlapi.v3arch.asyncio.AsyncCommandGenerator` or
    :py:class:`~pysnmp.hlapi.v3arch.asyncio.AsyncNotificationOriginator`
    and their derivatives for forming SNMP PDU and also adding new entries to
    Local Configuration Datastore (LCD) in order to support SNMPv1/v2c with
    SNMPv3 interoperability.

    See :RFC:`3411#section-4.1` for SNMP Context details.

    Parameters
    ----------
    contextEngineId : str
        Uniquely identifies an SNMP entity that may realize an instance of
        a MIB with a particular contextName (:RFC:`3411#section-3.3.2`).
        More frequently than not, ContextEngineID is the same as
        authoritative SnmpEngineID, however if SNMP Engine serves multiple
        SNMP Entities, their ContextEngineIDs would be distinct.
        Default is authoritative SNMP Engine ID.
    contextName : str
        Used to name an instance of MIB (:RFC:`3411#section-3.3.3`).
        Default is empty string.

    Examples
    --------
    >>> from pysnmp.hlapi.v3arch.asyncio import ContextData
    >>> ContextData()
    ContextData(contextEngineId=None, contextName='')
    >>> ContextData(OctetString(hexValue='01020ABBA0'))
    ContextData(contextEngineId=OctetString(hexValue='01020abba0'), contextName='')
    >>> ContextData(contextName='mycontext')
    ContextData(contextEngineId=None, contextName='mycontext')

    """

    def __init__(self, contextEngineId=None, contextName=b""):
        """Create a context data object."""
        self.contextEngineId = contextEngineId
        self.contextName = contextName

    def __repr__(self):
        """Return a string representation of the object."""
        return "{}(contextEngineId={!r}, contextName={!r})".format(
            self.__class__.__name__, self.contextEngineId, self.contextName
        )



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
# PySNMP MIB module TRANSPORT-ADDRESS-MIB (https://www.pysnmp.com/pysnmp)
# ASN.1 source file://asn1/TRANSPORT-ADDRESS-MIB
# Produced by pysmi-1.5.8 at Sat Nov  2 17:55:00 2024
# On host MacBook-Pro.local platform Darwin version 24.1.0 by user lextm
# Using Python version 3.12.0 (main, Nov 14 2023, 23:52:11) [Clang 15.0.0 (clang-1500.0.40.1)]

# IMPORTANT: this file contains customizations

import socket


from pysnmp import error


has_ipv6 = socket.has_ipv6

if hasattr(socket, "inet_ntop") and hasattr(socket, "inet_pton"):
    inet_ntop = socket.inet_ntop
    inet_pton = socket.inet_pton
else:
    import sys

    if sys.platform != "win32":
        from socket import inet_ntoa, inet_aton

        inet_ntop = lambda x, y: inet_ntoa(y)
        inet_pton = lambda x, y: inet_aton(y)
        has_ipv6 = False
    elif has_ipv6:
        import struct  # The case of old Python at old Windows

        def inet_pton(address_family, ip_string):
            if address_family == socket.AF_INET:
                return socket.inet_aton(ip_string)
            elif address_family != socket.AF_INET6:
                raise OSError(f"Unknown address family {address_family}")

            groups = ip_string.split(":")
            spaces = groups.count("")

            if "." in groups[-1]:
                groups[-1:] = [
                    "%x" % x for x in struct.unpack("!HH", socket.inet_aton(groups[-1]))
                ]

            if spaces == 1:
                idx = groups.index("")
                groups[idx : idx + 1] = ["0"] * (8 - len(groups) + 1)
            elif spaces == 2:
                zeros = ["0"] * (8 - len(groups) + 2)
                if ip_string.startswith("::"):
                    groups[:2] = zeros
                elif ip_string.endswith("::"):
                    groups[-2:] = zeros
                else:
                    raise OSError(f'Invalid IPv6 address: "{ip_string}"')
            elif spaces == 3:
                if ip_string != "::":
                    raise OSError(f'Invalid IPv6 address: "{ip_string}"')
                return "\x00" * 16
            elif spaces > 3:
                raise OSError(f'Invalid IPv6 address: "{ip_string}"')

            groups = [t for t in [int(t, 16) for t in groups] if t & 0xFFFF == t]

            if len(groups) != 8:
                raise OSError(f'Invalid IPv6 address: "{ip_string}"')

            return struct.pack("!8H", *groups)

        def inet_ntop(address_family, packed_ip):
            if address_family == socket.AF_INET:
                return socket.inet_ntop(packed_ip)
            elif address_family != socket.AF_INET6:
                raise OSError(f"Unknown address family {address_family}")

            if len(packed_ip) != 16:
                raise OSError("incorrect address length: %s" % len(packed_ip))

            groups = list(struct.unpack("!8H", packed_ip))

            cur_base = best_base = cur_len = best_len = -1

            for idx in range(8):
                if groups[idx]:
                    if cur_base != -1:
                        if best_base == -1 or cur_len > best_len:
                            best_base, best_len = cur_base, cur_len
                        cur_base = -1
                else:
                    if cur_base == -1:
                        cur_base, cur_len = idx, 1
                    else:
                        cur_len += 1

            if cur_base != -1:
                if best_base == -1 or cur_len > best_len:
                    best_base, best_len = cur_base, cur_len

            if best_base != -1 and best_len > 1:
                groups[best_base : best_base + best_len] = [":"]

            if groups[0] == ":":
                groups.insert(0, ":")
            if groups[-1] == ":":
                groups.append(":")

            f = lambda x: x != ":" and "%x" % x or ""

            return ":".join([f(x) for x in groups])


# Import base ASN.1 objects even if this MIB does not use it

(Integer,
 OctetString,
 ObjectIdentifier) = mibBuilder.import_symbols(
    "ASN1",
    "Integer",
    "OctetString",
    "ObjectIdentifier")

(NamedValues,) = mibBuilder.import_symbols(
    "ASN1-ENUMERATION",
    "NamedValues")
(ConstraintsIntersection,
 ConstraintsUnion,
 SingleValueConstraint,
 ValueRangeConstraint,
 ValueSizeConstraint) = mibBuilder.import_symbols(
    "ASN1-REFINEMENT",
    "ConstraintsIntersection",
    "ConstraintsUnion",
    "SingleValueConstraint",
    "ValueRangeConstraint",
    "ValueSizeConstraint")

# Import SMI symbols from the MIBs this MIB depends on

(ModuleCompliance,
 NotificationGroup) = mibBuilder.import_symbols(
    "SNMPv2-CONF",
    "ModuleCompliance",
    "NotificationGroup")

(Bits,
 Counter32,
 Counter64,
 Gauge32,
 Integer32,
 IpAddress,
 ModuleIdentity,
 MibIdentifier,
 NotificationType,
 ObjectIdentity,
 MibScalar,
 MibTable,
 MibTableRow,
 MibTableColumn,
 TimeTicks,
 Unsigned32,
 iso,
 mib_2) = mibBuilder.import_symbols(
    "SNMPv2-SMI",
    "Bits",
    "Counter32",
    "Counter64",
    "Gauge32",
    "Integer32",
    "IpAddress",
    "ModuleIdentity",
    "MibIdentifier",
    "NotificationType",
    "ObjectIdentity",
    "MibScalar",
    "MibTable",
    "MibTableRow",
    "MibTableColumn",
    "TimeTicks",
    "Unsigned32",
    "iso",
    "mib-2")

(DisplayString,
 TextualConvention) = mibBuilder.import_symbols(
    "SNMPv2-TC",
    "DisplayString",
    "TextualConvention")


# MODULE-IDENTITY

transportAddressMIB = ModuleIdentity(
    (1, 3, 6, 1, 2, 1, 100)
)
if mibBuilder.loadTexts:
    transportAddressMIB.setRevisions(
        ("2002-11-01 00:00",)
    )
if mibBuilder.loadTexts:
    transportAddressMIB.setLastUpdated("200211010000Z")
if mibBuilder.loadTexts:
    transportAddressMIB.setOrganization("IETF Operations and Management Area")
if mibBuilder.loadTexts:
    transportAddressMIB.setContactInfo("Juergen Schoenwaelder (Editor) TU Braunschweig Bueltenweg 74/75 38106 Braunschweig, Germany Phone: +49 531 391-3289 EMail: schoenw@ibr.cs.tu-bs.de Send comments to <mibs@ops.ietf.org>.")
if mibBuilder.loadTexts:
    transportAddressMIB.setDescription("This MIB module provides commonly used transport address definitions. Copyright (C) The Internet Society (2002). This version of this MIB module is part of RFC 3419; see the RFC itself for full legal notices.")


# Types definitions


# TEXTUAL-CONVENTIONS



class TransportDomain(TextualConvention, ObjectIdentifier):
    status = "current"
    if mibBuilder.loadTexts:
        description = "A value that represents a transport domain. Some possible values, such as transportDomainUdpIpv4, are defined in this module. Other possible values can be defined in other MIB modules."


class TransportAddressType(TextualConvention, Integer32):
    status = "current"
    subtypeSpec = Integer32.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        SingleValueConstraint(
            *(0,
              1,
              2,
              3,
              4,
              5,
              6,
              7,
              8,
              9,
              10,
              11,
              12,
              13,
              14,
              15,
              16)
        )
    )
    namedValues = NamedValues(
        *(("unknown", 0),
          ("udpIpv4", 1),
          ("udpIpv6", 2),
          ("udpIpv4z", 3),
          ("udpIpv6z", 4),
          ("tcpIpv4", 5),
          ("tcpIpv6", 6),
          ("tcpIpv4z", 7),
          ("tcpIpv6z", 8),
          ("sctpIpv4", 9),
          ("sctpIpv6", 10),
          ("sctpIpv4z", 11),
          ("sctpIpv6z", 12),
          ("local", 13),
          ("udpDns", 14),
          ("tcpDns", 15),
          ("sctpDns", 16))
    )

    if mibBuilder.loadTexts:
        description = "A value that represents a transport domain. This is the enumerated version of the transport domain registrations in this MIB module. The enumerated values have the following meaning: unknown(0) unknown transport address type udpIpv4(1) transportDomainUdpIpv4 udpIpv6(2) transportDomainUdpIpv6 udpIpv4z(3) transportDomainUdpIpv4z udpIpv6z(4) transportDomainUdpIpv6z tcpIpv4(5) transportDomainTcpIpv4 tcpIpv6(6) transportDomainTcpIpv6 tcpIpv4z(7) transportDomainTcpIpv4z tcpIpv6z(8) transportDomainTcpIpv6z sctpIpv4(9) transportDomainSctpIpv4 sctpIpv6(10) transportDomainSctpIpv6 sctpIpv4z(11) transportDomainSctpIpv4z sctpIpv6z(12) transportDomainSctpIpv6z local(13) transportDomainLocal udpDns(14) transportDomainUdpDns tcpDns(15) transportDomainTcpDns sctpDns(16) transportDomainSctpDns This textual convention can be used to represent transport domains in situations where a syntax of TransportDomain is unwieldy (for example, when used as an index). The usage of this textual convention implies that additional transport domains can only be supported by updating this MIB module. This extensibility restriction does not apply for the TransportDomain textual convention which allows MIB authors to define additional transport domains independently in other MIB modules."


class TransportAddress(TextualConvention, OctetString):
    status = "current"
    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 255),
    )

    if mibBuilder.loadTexts:
        description = "Denotes a generic transport address. A TransportAddress value is always interpreted within the context of a TransportAddressType or TransportDomain value. Every usage of the TransportAddress textual convention MUST specify the TransportAddressType or TransportDomain object which provides the context. Furthermore, MIB authors SHOULD define a separate TransportAddressType or TransportDomain object for each TransportAddress object. It is suggested that the TransportAddressType or TransportDomain is logically registered before the object(s) which use the TransportAddress textual convention if they appear in the same logical row. The value of a TransportAddress object must always be consistent with the value of the associated TransportAddressType or TransportDomain object. Attempts to set a TransportAddress object to a value which is inconsistent with the associated TransportAddressType or TransportDomain must fail with an inconsistentValue error. When this textual convention is used as a syntax of an index object, there may be issues with the limit of 128 sub-identifiers specified in SMIv2, STD 58. In this case, the OBJECT-TYPE declaration MUST include a 'SIZE' clause to limit the number of potential instance sub-identifiers."


class TransportAddressIPv4(TextualConvention, OctetString):
    status = "current"
    displayHint = "1d.1d.1d.1d:2d"
    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(6, 6),
    )
    fixedLength = 6

    if mibBuilder.loadTexts:
        description = "Represents a transport address consisting of an IPv4 address and a port number (as used for example by UDP, TCP and SCTP): octets contents encoding 1-4 IPv4 address network-byte order 5-6 port number network-byte order This textual convention SHOULD NOT be used directly in object definitions since it restricts addresses to a specific format. However, if it is used, it MAY be used either on its own or in conjunction with TransportAddressType or TransportDomain as a pair."

    def prettyIn(self, value):
        if isinstance(value, tuple):
            # Wild hack -- need to implement TextualConvention.prettyIn
            value = (
                inet_pton(socket.AF_INET, value[0])
                + bytes((((value[1] >> 8) & 0xFF),))
                + bytes(((value[1] & 0xFF),))
            )
        return OctetString.prettyIn(self, value)

    # Socket address syntax coercion
    def __asSocketAddress(self):
        if not hasattr(self, "__tuple_value"):
            v = self.asOctets()
            self.__tuple_value = (
                inet_ntop(socket.AF_INET, v[:4]),
                v[4] << 8 | v[5],
            )
        return self.__tuple_value

    def __iter__(self):
        return iter(self.__asSocketAddress())

    def __getitem__(self, item):
        return self.__asSocketAddress()[item]


class TransportAddressIPv6(TextualConvention, OctetString):
    status = "current"
    displayHint = "0a[2x:2x:2x:2x:2x:2x:2x:2x]0a:2d"
    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(18, 18),
    )
    fixedLength = 18

    if mibBuilder.loadTexts:
        description = "Represents a transport address consisting of an IPv6 address and a port number (as used for example by UDP, TCP and SCTP): octets contents encoding 1-16 IPv6 address network-byte order 17-18 port number network-byte order This textual convention SHOULD NOT be used directly in object definitions since it restricts addresses to a specific format. However, if it is used, it MAY be used either on its own or in conjunction with TransportAddressType or TransportDomain as a pair."

    def prettyIn(self, value):
        if not has_ipv6:
            raise error.PySnmpError("IPv6 not supported by platform")
        if isinstance(value, tuple):
            value = (
                inet_pton(socket.AF_INET6, value[0])
                + bytes((((value[1] >> 8) & 0xFF),))
                + bytes(((value[1] & 0xFF),))
            )
        return OctetString.prettyIn(self, value)

    # Socket address syntax coercion
    def __asSocketAddress(self):
        if not hasattr(self, "__tuple_value"):
            if not has_ipv6:
                raise error.PySnmpError("IPv6 not supported by platform")
            v = self.asOctets()
            self.__tuple_value = (
                inet_ntop(socket.AF_INET6, v[:16]),
                v[16] << 8 | v[17],
                0,  # flowinfo
                0,
            )  # scopeid
        return self.__tuple_value

    def __iter__(self):
        return iter(self.__asSocketAddress())

    def __getitem__(self, item):
        return self.__asSocketAddress()[item]


class TransportAddressIPv4z(TextualConvention, OctetString):
    status = "current"
    displayHint = "1d.1d.1d.1d%4d:2d"
    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(10, 10),
    )
    fixedLength = 10

    if mibBuilder.loadTexts:
        description = "Represents a transport address consisting of an IPv4 address, a zone index and a port number (as used for example by UDP, TCP and SCTP): octets contents encoding 1-4 IPv4 address network-byte order 5-8 zone index network-byte order 9-10 port number network-byte order This textual convention SHOULD NOT be used directly in object definitions since it restricts addresses to a specific format. However, if it is used, it MAY be used either on its own or in conjunction with TransportAddressType or TransportDomain as a pair."


class TransportAddressIPv6z(TextualConvention, OctetString):
    status = "current"
    displayHint = "0a[2x:2x:2x:2x:2x:2x:2x:2x%4d]0a:2d"
    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(22, 22),
    )
    fixedLength = 22

    if mibBuilder.loadTexts:
        description = "Represents a transport address consisting of an IPv6 address, a zone index and a port number (as used for example by UDP, TCP and SCTP): octets contents encoding 1-16 IPv6 address network-byte order 17-20 zone index network-byte order 21-22 port number network-byte order This textual convention SHOULD NOT be used directly in object definitions since it restricts addresses to a specific format. However, if it is used, it MAY be used either on its own or in conjunction with TransportAddressType or TransportDomain as a pair."


class TransportAddressLocal(TextualConvention, OctetString):
    status = "current"
    displayHint = "1a"
    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(1, 255),
    )

    if mibBuilder.loadTexts:
        description = "Represents a POSIX Local IPC transport address: octets contents encoding all POSIX Local IPC address string The Posix Local IPC transport domain subsumes UNIX domain sockets. This textual convention SHOULD NOT be used directly in object definitions since it restricts addresses to a specific format. However, if it is used, it MAY be used either on its own or in conjunction with TransportAddressType or TransportDomain as a pair. When this textual convention is used as a syntax of an index object, there may be issues with the limit of 128 sub-identifiers specified in SMIv2, STD 58. In this case, the OBJECT-TYPE declaration MUST include a 'SIZE' clause to limit the number of potential instance sub-identifiers."
    if mibBuilder.loadTexts:
        reference = "Protocol Independent Interfaces (IEEE POSIX 1003.1g)"


class TransportAddressDns(TextualConvention, OctetString):
    status = "current"
    displayHint = "1a"
    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(1, 255),
    )

    if mibBuilder.loadTexts:
        description = "Represents a DNS domain name followed by a colon ':' (ASCII character 0x3A) and a port number in ASCII. The name SHOULD be fully qualified whenever possible. Values of this textual convention are not directly useable as transport-layer addressing information, and require runtime resolution. As such, applications that write them must be prepared for handling errors if such values are not supported, or cannot be resolved (if resolution occurs at the time of the management operation). The DESCRIPTION clause of TransportAddress objects that may have TransportAddressDns values must fully describe how (and when) such names are to be resolved to IP addresses and vice versa. This textual convention SHOULD NOT be used directly in object definitions since it restricts addresses to a specific format. However, if it is used, it MAY be used either on its own or in conjunction with TransportAddressType or TransportDomain as a pair. When this textual convention is used as a syntax of an index object, there may be issues with the limit of 128 sub-identifiers specified in SMIv2, STD 58. In this case, the OBJECT-TYPE declaration MUST include a 'SIZE' clause to limit the number of potential instance sub-identifiers."


# MIB Managed Objects in the order of their OIDs

_TransportDomains_ObjectIdentity = ObjectIdentity
transportDomains = _TransportDomains_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1)
)
_TransportDomainUdpIpv4_ObjectIdentity = ObjectIdentity
transportDomainUdpIpv4 = _TransportDomainUdpIpv4_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 1)
)
if mibBuilder.loadTexts:
    transportDomainUdpIpv4.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainUdpIpv4.setDescription("The UDP over IPv4 transport domain. The corresponding transport address is of type TransportAddressIPv4 for global IPv4 addresses.")
_TransportDomainUdpIpv6_ObjectIdentity = ObjectIdentity
transportDomainUdpIpv6 = _TransportDomainUdpIpv6_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 2)
)
if mibBuilder.loadTexts:
    transportDomainUdpIpv6.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainUdpIpv6.setDescription("The UDP over IPv6 transport domain. The corresponding transport address is of type TransportAddressIPv6 for global IPv6 addresses.")
_TransportDomainUdpIpv4z_ObjectIdentity = ObjectIdentity
transportDomainUdpIpv4z = _TransportDomainUdpIpv4z_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 3)
)
if mibBuilder.loadTexts:
    transportDomainUdpIpv4z.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainUdpIpv4z.setDescription("The UDP over IPv4 transport domain. The corresponding transport address is of type TransportAddressIPv4z for scoped IPv4 addresses with a zone index.")
_TransportDomainUdpIpv6z_ObjectIdentity = ObjectIdentity
transportDomainUdpIpv6z = _TransportDomainUdpIpv6z_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 4)
)
if mibBuilder.loadTexts:
    transportDomainUdpIpv6z.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainUdpIpv6z.setDescription("The UDP over IPv6 transport domain. The corresponding transport address is of type TransportAddressIPv6z for scoped IPv6 addresses with a zone index.")
_TransportDomainTcpIpv4_ObjectIdentity = ObjectIdentity
transportDomainTcpIpv4 = _TransportDomainTcpIpv4_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 5)
)
if mibBuilder.loadTexts:
    transportDomainTcpIpv4.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainTcpIpv4.setDescription("The TCP over IPv4 transport domain. The corresponding transport address is of type TransportAddressIPv4 for global IPv4 addresses.")
_TransportDomainTcpIpv6_ObjectIdentity = ObjectIdentity
transportDomainTcpIpv6 = _TransportDomainTcpIpv6_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 6)
)
if mibBuilder.loadTexts:
    transportDomainTcpIpv6.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainTcpIpv6.setDescription("The TCP over IPv6 transport domain. The corresponding transport address is of type TransportAddressIPv6 for global IPv6 addresses.")
_TransportDomainTcpIpv4z_ObjectIdentity = ObjectIdentity
transportDomainTcpIpv4z = _TransportDomainTcpIpv4z_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 7)
)
if mibBuilder.loadTexts:
    transportDomainTcpIpv4z.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainTcpIpv4z.setDescription("The TCP over IPv4 transport domain. The corresponding transport address is of type TransportAddressIPv4z for scoped IPv4 addresses with a zone index.")
_TransportDomainTcpIpv6z_ObjectIdentity = ObjectIdentity
transportDomainTcpIpv6z = _TransportDomainTcpIpv6z_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 8)
)
if mibBuilder.loadTexts:
    transportDomainTcpIpv6z.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainTcpIpv6z.setDescription("The TCP over IPv6 transport domain. The corresponding transport address is of type TransportAddressIPv6z for scoped IPv6 addresses with a zone index.")
_TransportDomainSctpIpv4_ObjectIdentity = ObjectIdentity
transportDomainSctpIpv4 = _TransportDomainSctpIpv4_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 9)
)
if mibBuilder.loadTexts:
    transportDomainSctpIpv4.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainSctpIpv4.setDescription("The SCTP over IPv4 transport domain. The corresponding transport address is of type TransportAddressIPv4 for global IPv4 addresses. This transport domain usually represents the primary address on multihomed SCTP endpoints.")
_TransportDomainSctpIpv6_ObjectIdentity = ObjectIdentity
transportDomainSctpIpv6 = _TransportDomainSctpIpv6_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 10)
)
if mibBuilder.loadTexts:
    transportDomainSctpIpv6.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainSctpIpv6.setDescription("The SCTP over IPv6 transport domain. The corresponding transport address is of type TransportAddressIPv6 for global IPv6 addresses. This transport domain usually represents the primary address on multihomed SCTP endpoints.")
_TransportDomainSctpIpv4z_ObjectIdentity = ObjectIdentity
transportDomainSctpIpv4z = _TransportDomainSctpIpv4z_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 11)
)
if mibBuilder.loadTexts:
    transportDomainSctpIpv4z.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainSctpIpv4z.setDescription("The SCTP over IPv4 transport domain. The corresponding transport address is of type TransportAddressIPv4z for scoped IPv4 addresses with a zone index. This transport domain usually represents the primary address on multihomed SCTP endpoints.")
_TransportDomainSctpIpv6z_ObjectIdentity = ObjectIdentity
transportDomainSctpIpv6z = _TransportDomainSctpIpv6z_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 12)
)
if mibBuilder.loadTexts:
    transportDomainSctpIpv6z.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainSctpIpv6z.setDescription("The SCTP over IPv6 transport domain. The corresponding transport address is of type TransportAddressIPv6z for scoped IPv6 addresses with a zone index. This transport domain usually represents the primary address on multihomed SCTP endpoints.")
_TransportDomainLocal_ObjectIdentity = ObjectIdentity
transportDomainLocal = _TransportDomainLocal_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 13)
)
if mibBuilder.loadTexts:
    transportDomainLocal.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainLocal.setDescription("The Posix Local IPC transport domain. The corresponding transport address is of type TransportAddressLocal. The Posix Local IPC transport domain incorporates the well-known UNIX domain sockets.")
_TransportDomainUdpDns_ObjectIdentity = ObjectIdentity
transportDomainUdpDns = _TransportDomainUdpDns_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 14)
)
if mibBuilder.loadTexts:
    transportDomainUdpDns.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainUdpDns.setDescription("The UDP transport domain using fully qualified domain names. The corresponding transport address is of type TransportAddressDns.")
_TransportDomainTcpDns_ObjectIdentity = ObjectIdentity
transportDomainTcpDns = _TransportDomainTcpDns_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 15)
)
if mibBuilder.loadTexts:
    transportDomainTcpDns.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainTcpDns.setDescription("The TCP transport domain using fully qualified domain names. The corresponding transport address is of type TransportAddressDns.")
_TransportDomainSctpDns_ObjectIdentity = ObjectIdentity
transportDomainSctpDns = _TransportDomainSctpDns_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 100, 1, 16)
)
if mibBuilder.loadTexts:
    transportDomainSctpDns.setStatus("current")
if mibBuilder.loadTexts:
    transportDomainSctpDns.setDescription("The SCTP transport domain using fully qualified domain names. The corresponding transport address is of type TransportAddressDns.")

# Managed Objects groups


# Notification objects


# Notifications groups


# Agent capabilities


# Module compliance


# Export all MIB objects to the MIB builder

mibBuilder.export_symbols(
    "TRANSPORT-ADDRESS-MIB",
    **{"TransportDomain": TransportDomain,
       "TransportAddressType": TransportAddressType,
       "TransportAddress": TransportAddress,
       "TransportAddressIPv4": TransportAddressIPv4,
       "TransportAddressIPv6": TransportAddressIPv6,
       "TransportAddressIPv4z": TransportAddressIPv4z,
       "TransportAddressIPv6z": TransportAddressIPv6z,
       "TransportAddressLocal": TransportAddressLocal,
       "TransportAddressDns": TransportAddressDns,
       "transportAddressMIB": transportAddressMIB,
       "transportDomains": transportDomains,
       "transportDomainUdpIpv4": transportDomainUdpIpv4,
       "transportDomainUdpIpv6": transportDomainUdpIpv6,
       "transportDomainUdpIpv4z": transportDomainUdpIpv4z,
       "transportDomainUdpIpv6z": transportDomainUdpIpv6z,
       "transportDomainTcpIpv4": transportDomainTcpIpv4,
       "transportDomainTcpIpv6": transportDomainTcpIpv6,
       "transportDomainTcpIpv4z": transportDomainTcpIpv4z,
       "transportDomainTcpIpv6z": transportDomainTcpIpv6z,
       "transportDomainSctpIpv4": transportDomainSctpIpv4,
       "transportDomainSctpIpv6": transportDomainSctpIpv6,
       "transportDomainSctpIpv4z": transportDomainSctpIpv4z,
       "transportDomainSctpIpv6z": transportDomainSctpIpv6z,
       "transportDomainLocal": transportDomainLocal,
       "transportDomainUdpDns": transportDomainUdpDns,
       "transportDomainTcpDns": transportDomainTcpDns,
       "transportDomainSctpDns": transportDomainSctpDns,
       "PYSNMP_MODULE_ID": transportAddressMIB}
)



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
#
# PySNMP MIB module SNMP-MPD-MIB (https://www.pysnmp.com/pysnmp)
# ASN.1 source file://asn1/SNMP-MPD-MIB
# Produced by pysmi-1.5.8 at Sat Nov  2 15:25:32 2024
# On host MacBook-Pro.local platform Darwin version 24.1.0 by user lextm
# Using Python version 3.12.0 (main, Nov 14 2023, 23:52:11) [Clang 15.0.0 (clang-1500.0.40.1)]
#

# Import base ASN.1 objects even if this MIB does not use it

(Integer,
 OctetString,
 ObjectIdentifier) = mibBuilder.import_symbols(
    "ASN1",
    "Integer",
    "OctetString",
    "ObjectIdentifier")

(NamedValues,) = mibBuilder.import_symbols(
    "ASN1-ENUMERATION",
    "NamedValues")
(ConstraintsIntersection,
 ConstraintsUnion,
 SingleValueConstraint,
 ValueRangeConstraint,
 ValueSizeConstraint) = mibBuilder.import_symbols(
    "ASN1-REFINEMENT",
    "ConstraintsIntersection",
    "ConstraintsUnion",
    "SingleValueConstraint",
    "ValueRangeConstraint",
    "ValueSizeConstraint")

# Import SMI symbols from the MIBs this MIB depends on

(ModuleCompliance,
 NotificationGroup,
 ObjectGroup) = mibBuilder.import_symbols(
    "SNMPv2-CONF",
    "ModuleCompliance",
    "NotificationGroup",
    "ObjectGroup")

(Bits,
 Counter32,
 Counter64,
 Gauge32,
 Integer32,
 IpAddress,
 ModuleIdentity,
 MibIdentifier,
 NotificationType,
 ObjectIdentity,
 MibScalar,
 MibTable,
 MibTableRow,
 MibTableColumn,
 TimeTicks,
 Unsigned32,
 iso,
 snmpModules) = mibBuilder.import_symbols(
    "SNMPv2-SMI",
    "Bits",
    "Counter32",
    "Counter64",
    "Gauge32",
    "Integer32",
    "IpAddress",
    "ModuleIdentity",
    "MibIdentifier",
    "NotificationType",
    "ObjectIdentity",
    "MibScalar",
    "MibTable",
    "MibTableRow",
    "MibTableColumn",
    "TimeTicks",
    "Unsigned32",
    "iso",
    "snmpModules")

(DisplayString,
 TextualConvention) = mibBuilder.import_symbols(
    "SNMPv2-TC",
    "DisplayString",
    "TextualConvention")


# MODULE-IDENTITY

snmpMPDMIB = ModuleIdentity(
    (1, 3, 6, 1, 6, 3, 11)
)
if mibBuilder.loadTexts:
    snmpMPDMIB.setRevisions(
        ("2002-10-14 00:00",
         "1999-05-04 16:36",
         "1997-09-30 00:00")
    )
if mibBuilder.loadTexts:
    snmpMPDMIB.setLastUpdated("200210140000Z")
if mibBuilder.loadTexts:
    snmpMPDMIB.setOrganization("SNMPv3 Working Group")
if mibBuilder.loadTexts:
    snmpMPDMIB.setContactInfo("WG-EMail: snmpv3@lists.tislabs.com Subscribe: snmpv3-request@lists.tislabs.com Co-Chair: Russ Mundy Network Associates Laboratories postal: 15204 Omega Drive, Suite 300 Rockville, MD 20850-4601 USA EMail: mundy@tislabs.com phone: +1 301-947-7107 Co-Chair & Co-editor: David Harrington Enterasys Networks postal: 35 Industrial Way P. O. Box 5005 Rochester NH 03866-5005 USA EMail: dbh@enterasys.com phone: +1 603-337-2614 Co-editor: Jeffrey Case SNMP Research, Inc. postal: 3001 Kimberlin Heights Road Knoxville, TN 37920-9716 USA EMail: case@snmp.com phone: +1 423-573-1434 Co-editor: Randy Presuhn BMC Software, Inc. postal: 2141 North First Street San Jose, CA 95131 USA EMail: randy_presuhn@bmc.com phone: +1 408-546-1006 Co-editor: Bert Wijnen Lucent Technologies postal: Schagen 33 3461 GL Linschoten Netherlands EMail: bwijnen@lucent.com phone: +31 348-680-485 ")
if mibBuilder.loadTexts:
    snmpMPDMIB.setDescription("The MIB for Message Processing and Dispatching Copyright (C) The Internet Society (2002). This version of this MIB module is part of RFC 3412; see the RFC itself for full legal notices. ")


# Types definitions


# TEXTUAL-CONVENTIONS



# MIB Managed Objects in the order of their OIDs

_SnmpMPDAdmin_ObjectIdentity = ObjectIdentity
snmpMPDAdmin = _SnmpMPDAdmin_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 11, 1)
)
_SnmpMPDMIBObjects_ObjectIdentity = ObjectIdentity
snmpMPDMIBObjects = _SnmpMPDMIBObjects_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 11, 2)
)
_SnmpMPDStats_ObjectIdentity = ObjectIdentity
snmpMPDStats = _SnmpMPDStats_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 11, 2, 1)
)
_SnmpUnknownSecurityModels_Type = Counter32
_SnmpUnknownSecurityModels_Object = MibScalar
snmpUnknownSecurityModels = _SnmpUnknownSecurityModels_Object(
    (1, 3, 6, 1, 6, 3, 11, 2, 1, 1),
    _SnmpUnknownSecurityModels_Type()
)
snmpUnknownSecurityModels.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpUnknownSecurityModels.setStatus("current")
if mibBuilder.loadTexts:
    snmpUnknownSecurityModels.setDescription("The total number of packets received by the SNMP engine which were dropped because they referenced a securityModel that was not known to or supported by the SNMP engine. ")
_SnmpInvalidMsgs_Type = Counter32
_SnmpInvalidMsgs_Object = MibScalar
snmpInvalidMsgs = _SnmpInvalidMsgs_Object(
    (1, 3, 6, 1, 6, 3, 11, 2, 1, 2),
    _SnmpInvalidMsgs_Type()
)
snmpInvalidMsgs.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInvalidMsgs.setStatus("current")
if mibBuilder.loadTexts:
    snmpInvalidMsgs.setDescription("The total number of packets received by the SNMP engine which were dropped because there were invalid or inconsistent components in the SNMP message. ")
_SnmpUnknownPDUHandlers_Type = Counter32
_SnmpUnknownPDUHandlers_Object = MibScalar
snmpUnknownPDUHandlers = _SnmpUnknownPDUHandlers_Object(
    (1, 3, 6, 1, 6, 3, 11, 2, 1, 3),
    _SnmpUnknownPDUHandlers_Type()
)
snmpUnknownPDUHandlers.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpUnknownPDUHandlers.setStatus("current")
if mibBuilder.loadTexts:
    snmpUnknownPDUHandlers.setDescription("The total number of packets received by the SNMP engine which were dropped because the PDU contained in the packet could not be passed to an application responsible for handling the pduType, e.g. no SNMP application had registered for the proper combination of the contextEngineID and the pduType. ")
_SnmpMPDMIBConformance_ObjectIdentity = ObjectIdentity
snmpMPDMIBConformance = _SnmpMPDMIBConformance_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 11, 3)
)
_SnmpMPDMIBCompliances_ObjectIdentity = ObjectIdentity
snmpMPDMIBCompliances = _SnmpMPDMIBCompliances_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 11, 3, 1)
)
_SnmpMPDMIBGroups_ObjectIdentity = ObjectIdentity
snmpMPDMIBGroups = _SnmpMPDMIBGroups_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 11, 3, 2)
)

# Managed Objects groups

snmpMPDGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 11, 3, 2, 1)
)
snmpMPDGroup.setObjects(
      *(("SNMP-MPD-MIB", "snmpUnknownSecurityModels"),
        ("SNMP-MPD-MIB", "snmpInvalidMsgs"),
        ("SNMP-MPD-MIB", "snmpUnknownPDUHandlers"))
)
if mibBuilder.loadTexts:
    snmpMPDGroup.setStatus("current")
if mibBuilder.loadTexts:
    snmpMPDGroup.setDescription("A collection of objects providing for remote monitoring of the SNMP Message Processing and Dispatching process. ")


# Notification objects


# Notifications groups


# Agent capabilities


# Module compliance

snmpMPDCompliance = ModuleCompliance(
    (1, 3, 6, 1, 6, 3, 11, 3, 1, 1)
)
snmpMPDCompliance.setObjects(
    ("SNMP-MPD-MIB", "snmpMPDGroup")
)
if mibBuilder.loadTexts:
    snmpMPDCompliance.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    snmpMPDCompliance.setDescription("The compliance statement for SNMP entities which implement the SNMP-MPD-MIB. ")


# Export all MIB objects to the MIB builder

mibBuilder.export_symbols(
    "SNMP-MPD-MIB",
    **{"snmpMPDMIB": snmpMPDMIB,
       "snmpMPDAdmin": snmpMPDAdmin,
       "snmpMPDMIBObjects": snmpMPDMIBObjects,
       "snmpMPDStats": snmpMPDStats,
       "snmpUnknownSecurityModels": snmpUnknownSecurityModels,
       "snmpInvalidMsgs": snmpInvalidMsgs,
       "snmpUnknownPDUHandlers": snmpUnknownPDUHandlers,
       "snmpMPDMIBConformance": snmpMPDMIBConformance,
       "snmpMPDMIBCompliances": snmpMPDMIBCompliances,
       "snmpMPDCompliance": snmpMPDCompliance,
       "snmpMPDMIBGroups": snmpMPDMIBGroups,
       "snmpMPDGroup": snmpMPDGroup,
       "PYSNMP_MODULE_ID": snmpMPDMIB}
)



"""
This module initializes the versioning information for the pysnmp package.

Attributes:
    __version__ (str): The current version of the pysnmp package.
    main_version (str): The main version string, used to prevent semantic release from updating the version in multiple places.
    version (tuple): A tuple representation of the version, where each part of the version string is converted to an integer.
    major_version_id (int): The major version number extracted from the version tuple.

Notes:
    - For backward compatibility, if the version string contains "beta", the string part is removed before converting to a tuple.
"""
# http://www.python.org/dev/peps/pep-0396/
__version__ = "7.1.19"
# another variable is required to prevent semantic release from updating version in more than one place
main_version = __version__
# backward compatibility
# for beta versions, integer casting throws an exception, so string part must be cut off
if "beta" in __version__:
    main_version = __version__.split("-beta")[0]
version = tuple(int(x) for x in main_version.split("."))
major_version_id = version[0]
